import { Readable } from 'stream';
import {
    LanguageCode,
    MediaEncoding
} from '@aws-sdk/client-transcribe-streaming';

/**
 * Represents configurable options for FFmpeg media processing tasks.
 */
interface FFmpegOptions {
    /** Specifies the codec to be used for encoding or decoding */
    codec?: string;
    /** Defines the audio sample rate in Hz */
    sampleRate?: number;
    /** Sets the number of audio channels */
    channels?: number;
    /** Determines the output media format */
    format?: string;
}

/**
 * Converts an audio stream to PCM (Pulse-Code Modulation) format.
 *
 * This function takes an audio input stream and converts it into PCM format
 * using specified configurations.
 *
 * @param inputStream - A readable stream containing the audio data to be converted
 * @param options - An optional parameter specifying FFmpeg options to configure the conversion process
 * @returns A promise that resolves to a readable stream containing the PCM converted audio data
 */
export function convertAudioToPcm(
    inputStream: Readable,
    options?: FFmpegOptions,
): Promise<Readable>;

/**
 * Transcribes audio data from a readable PCM audio stream into a text string using AWS Transcribe Streaming Service.
 *
 * @param accessKeyId - The AWS access key ID to authenticate the Transcribe client
 * @param secretAccessKey - The AWS secret access key to authenticate the Transcribe client
 * @param region - The AWS region where the Transcribe service is hosted
 * @param audioStream - A readable stream of PCM audio data to be transcribed
 * @param languageCode - The language code for the transcription (default is 'pt-BR')
 * @param mediaEncoding - The encoding format of the input audio data (default is 'pcm')
 * @param sampleRate - The sample rate (in Hz) of the input audio data (default is 8000Hz)
 * @returns A promise that resolves to the transcribed text
 */
export function transcribeAudioStreamToText(
    accessKeyId: string,
    secretAccessKey: string,
    region: string,
    audioStream: Readable,
    languageCode?: LanguageCode,
    mediaEncoding?: MediaEncoding,
    sampleRate?: number
): Promise<string>;